(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("DashboardHelper", ["$q", "AppConstants", "TranslationService", "FieldLabels", "PicklistLabels",
            "RecordTypeLabels", MedicalDashboardHelper])
    ;

    function MedicalDashboardHelper($q, appConstants, translationService, fieldLabels, picklistLabels,
                                    recordTypeLabels) {
        var helper = this;

        helper.excludeUnreachableFieldFromQueries = function(queries, unreachableObjects) {
            for(var key in queries) {
                if(Object.prototype.hasOwnProperty.call(queries, key)) {
                    var objectName = queries[key].object;
                    var queryFields = queries[key].fields;
                    var unreachableFields = unreachableObjects[objectName];
                    if(!!unreachableFields && unreachableFields.length > 0) {
                        // exclude query field(s) that is not accessible
                        console.warn('Excluding query field(s): ' + unreachableFields.join(',') + ' from ' + objectName);
                        queries[key].fields = _.difference(queryFields, unreachableFields);
                    }
                }
            }
        };

        helper.getLabels = function(unreachableObjects) {
            var deferred = $q.defer();
            var objectLabelDeferred = $q.defer();
            var fieldLabelDeferred = $q.defer();
            var picklistLabelDeferred = $q.defer();
            var recordTypeLabelDeferred = $q.defer();
            var promises = [
                objectLabelDeferred.promise,
                fieldLabelDeferred.promise,
                picklistLabelDeferred.promise,
                recordTypeLabelDeferred.promise
            ];

            var fieldLabelToTranslate = angular.copy(fieldLabels);
            var picklistLabelToTranslate = angular.copy(picklistLabels);
            var recordTypeLabelToTranslate = _.differenceBy(recordTypeLabels, _.keys(unreachableObjects));
            for(var key in unreachableObjects) {
                if(Object.prototype.hasOwnProperty.call(unreachableObjects, key)) {
                    var unreachableFields = unreachableObjects[key];
                    if(unreachableFields.length > 0) {
                        fieldLabelToTranslate[key] = _.difference(fieldLabelToTranslate[key], unreachableFields);
                    } else {
                        delete fieldLabelToTranslate[key];
                        delete picklistLabelToTranslate[key];
                    }
                }
            }

            translationService.getObjectLabels(fieldLabelToTranslate).then(function(resp) {
                objectLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(objectLabelDeferred));
            translationService.getFieldLabels(fieldLabelToTranslate).then(function(resp) {
                fieldLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(fieldLabelDeferred));
            translationService.getPicklistLabels(picklistLabelToTranslate).then(function(resp) {
                picklistLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(picklistLabelDeferred));
            translationService.getRecordTypeLabels(recordTypeLabelToTranslate).then(function(resp) {
                recordTypeLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(recordTypeLabelDeferred));

            $q.all(promises).then(function(comboResp) {
                deferred.resolve({
                    objectLabels: comboResp[0],
                    fieldLabels: comboResp[1],
                    picklistLabels: comboResp[2],
                    recordTypeLabels: comboResp[3]
                });
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        helper.okToEnableTile = function(tileName, unreachableObjects, isTM2) {
            var enable = true;

            if(tileName === appConstants.SENT_MESSAGE) {
                enable = !angular.isDefined(unreachableObjects.Sent_Message_vod__c) || unreachableObjects.Sent_Message_vod__c.length > 0;
            }
            else if(tileName === appConstants.CONSENT) {
                enable = (!angular.isDefined(unreachableObjects.Multichannel_Consent_vod__c) || unreachableObjects.Multichannel_Consent_vod__c.length > 0)
                    && !angular.isDefined(getCorrespondingTerritory(unreachableObjects, isTM2))
                    && !angular.isDefined(getCorrespondingUserTerritory(unreachableObjects, isTM2));
            }

            if(!enable) {
                console.warn(tileName + " is not enabled");
            }
            return enable;
        };

        function genericQueryErrorHandler(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        }

        function getCorrespondingTerritory(unreachableObjects, isTM2) {
            if (isTM2) {
                return unreachableObjects.Territory2;
            }

            return unreachableObjects.Territory;
        }

        function getCorrespondingUserTerritory(unreachableObjects, isTM2) {
            if (isTM2) {
                return unreachableObjects.UserTerritory2Association;
            }

            return unreachableObjects.UserTerritory;
        }

        return helper;
    }
}());
